/*******************************************************************************
 * Copyright (c) 2004, 2005 Sybase, Inc. and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sybase, Inc. - initial API and implementation
 *******************************************************************************/
package org.eclipse.jst.jsf.facesconfig.common.dialogfield;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.eclipse.jst.jsf.facesconfig.common.utils.ColorUtil;
import org.eclipse.jst.jsf.facesconfig.common.widget.StyleCombo;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.widgets.FormToolkit;


/**
 * This class provides a common color choice DialogFiled.
 * 
 * @author zpzhang
 */
public class ColorButtonDialogField extends StyleComboDialogField {
	// color selection button
	private Button button;

	// the button enable flag
	private boolean buttonEnabled;

	// the current color
	private Color color;

	// the current color RGB
	private RGB colorValue;

	// the combo control
	private StyleCombo combo;

	// when no color is set or selected, using the empty image
	private Image emptyImage;

	// size of color image
	private Point extent;

	// the color image
	private Image image;

	public ColorButtonDialogField() {
		this(SWT.NONE);
	}

	public ColorButtonDialogField(int flags) {
		super(flags);
		buttonEnabled = true;
	}

	private void buttonPressed() {
		ColorDialog colorDialog = new ColorDialog(button.getShell());
		colorDialog.setRGB(colorValue);
		RGB newColor = colorDialog.open();
		if (newColor != null && !newColor.equals(colorValue)) {
			colorValue = newColor;
			setText(ColorUtil.getStringColor(newColor));
			updateColorImage();
		}
	}

	/**
	 * Compute the size of the image to be displayed.
	 * 
	 * @param window -
	 *            the window used to calculate
	 * @return <code>Point</code>
	 */
	private Point computeImageSize(Control window) {
		GC gc = new GC(window);
		Font f = button.getFont();
		gc.setFont(f);
		int height = gc.getFontMetrics().getHeight();

		Point p = new Point(height * 3 - 6, height - 2);
		gc.dispose();
		return p;
	}

	private void disposeResources() {
		if (image != null) {
			image.dispose();
			image = null;
		}
		if (emptyImage != null) {
			emptyImage.dispose();
			emptyImage = null;
		}
		if (color != null) {
			color.dispose();
			color = null;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.DialogField#doFillIntoGrid(org.eclipse.ui.forms.widgets.FormToolkit,
	 *      org.eclipse.swt.widgets.Composite, int)
	 */
	public Control[] doFillIntoGrid(FormToolkit toolkit, Composite parent,
			int nColumns) {
		assertEnoughColumns(nColumns);

		Control requiredLabel = getRequiredLabelControl(toolkit, parent);
		requiredLabel.setLayoutData(gridDataForLabel(1));

		Control label = getLabelControl(toolkit, parent);
		label.setLayoutData(gridDataForLabel(1));

		StyleCombo combo = getComboControl(toolkit, parent);
		combo.setLayoutData(gridDataForCombo(nColumns - 3));

		button = getChangeControl(toolkit, parent);
		button.setLayoutData(gridDataForButton(1));

		return new Control[] { requiredLabel, label, combo, button };
	}

	/**
	 * Sets the enable state of the button.
	 */
	public void enableButton(boolean enable) {
		if (isOkToUse(button)) {
			button.setEnabled(isEnabled() && enable);
		}
		buttonEnabled = enable;
	}

	/**
	 * Creates or returns the created buttom widget.
	 * 
	 * @param parent
	 *            The parent composite or <code>null</code> if the widget has
	 *            already been created.
	 */
	public Button getChangeControl(FormToolkit toolkit, Composite parent) {
		if (button == null) {
			assertCompositeNotNull(parent);
			if (toolkit != null) {
				button = toolkit.createButton(parent, "", SWT.PUSH);
			} else {
				button = new Button(parent, SWT.PUSH);
			}
			button.setEnabled(isEnabled() && buttonEnabled);

			button.addSelectionListener(new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent e) {
					buttonPressed();
				}

				public void widgetSelected(SelectionEvent e) {
					buttonPressed();
				}
			});

			button.addDisposeListener(new DisposeListener() {
				public void widgetDisposed(DisposeEvent event) {
					disposeResources();
				}
			});

			extent = computeImageSize(parent);
			image = new Image(parent.getDisplay(), extent.x, extent.y);
			emptyImage = new Image(parent.getDisplay(), extent.x, extent.y);
			initEmptyImage(button);
			updateColorImage();
		}

		return button;
	}

	private String[] getColorList() {
		Map map = ColorUtil.getCSSColorMap();
		List list = new ArrayList(map.keySet());
		Collections.sort(list);
		return (String[]) list.toArray(new String[list.size()]);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.StringDialogField#getTextControl(org.eclipse.ui.forms.widgets.FormToolkit,
	 *      org.eclipse.swt.widgets.Composite)
	 */
	public StyleCombo getComboControl(FormToolkit toolkit, Composite parent) {

		if (combo == null) {
			combo = super.getComboControl(toolkit, parent);

			combo.addSelectionListener(new SelectionListener() {

				public void widgetDefaultSelected(SelectionEvent e) {
					updateImageAfterChanged();
				}

				public void widgetSelected(SelectionEvent e) {
					updateImageAfterChanged();
				}
			});
			combo.addFocusListener(new FocusAdapter() {
				public void focusLost(FocusEvent e) {
					updateImageAfterChanged();
				}
			});
			combo.setItems(getColorList());
		}
		return combo;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.DialogField#getNumberOfControls()
	 */
	public int getNumberOfControls() {
		return 4;
	}

	public StyleCombo getTextControl(FormToolkit toolkit, Composite parent) {
		return getComboControl(toolkit, parent);
	}

	protected GridData gridDataForButton(int span) {
		GridData gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		gd.grabExcessHorizontalSpace = false;
		gd.horizontalSpan = span;
		gd.widthHint = LayoutUtil.getButtonWidthHint(button);
		gd.heightHint = combo.computeSize(SWT.DEFAULT, SWT.DEFAULT).y;
		return gd;
	}

	private void initEmptyImage(Control parent) {
		Color transparentColor = parent.getForeground();
		ImageData imageData = emptyImage.getImageData();
		imageData.transparentPixel = 0;
		GC gc = new GC(emptyImage);
		gc.setBackground(transparentColor);
		gc.fillRectangle(button.getBounds());
		gc.dispose();
	}

	/**
	 * Set the current color value and update the control.
	 * 
	 * @param rgb
	 *            The new color.
	 */
	public void setColorValue(RGB rgb) {
		colorValue = rgb;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.ComboDialogField#setTextWithoutUpdate(java.lang.String)
	 */
	public void setTextWithoutUpdate(String text) {
		super.setTextWithoutUpdate(text);
		updateImageAfterChanged();
	}

	/**
	 * Update the image being displayed on the button using the current color
	 * setting.
	 */
	protected void updateColorImage() {
		if (colorValue == null) {
			button.setImage(emptyImage);
			return;
		}

		Display display = button.getDisplay();
		GC gc = new GC(image);
		gc.setForeground(display.getSystemColor(SWT.COLOR_BLACK));
		gc.drawRectangle(0, 0, extent.x - 1, extent.y - 1);
		if (color != null) {
			color.dispose();
		}
		color = new Color(display, colorValue);
		gc.setBackground(color);
		gc.fillRectangle(1, 1, extent.x - 2, extent.y - 2);
		gc.dispose();
		button.setImage(image);
	}

	/*
	 * @see DialogField#updateEnableState
	 */
	protected void updateEnableState() {
		super.updateEnableState();
		if (isOkToUse(button)) {
			button.setEnabled(isEnabled() && buttonEnabled);
		}
	}

	private void updateImageAfterChanged() {
		String newColor = getText().trim();
		setColorValue(ColorUtil.getColorRGB(newColor));
		updateColorImage();
	}
}
