/*******************************************************************************
 * Copyright (c) 2004, 2005 Sybase, Inc. and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sybase, Inc. - initial API and implementation
 *******************************************************************************/
package org.eclipse.jst.jsf.facesconfig.common.dialogfield;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.forms.widgets.FormToolkit;

/**
 * This will display a labal and a group of radio buttons. The group of radio
 * buttons will be layed out horizontally use a RowLayout, and them as a group
 * will use one cell in the GridLayout.
 * 
 * Whenever the radios selection change will fire both dialogFieldChanged() and
 * dialogFieldApplied() event.
 * 
 * @author Yang Liu
 */
public class RadiosDialogField extends DialogFieldBase {
	final static private String INDEXKEY = "INDEX";

	private Composite group;

	private String[] items;

	private Button[] button;

	private int selectIndex = -1;

	private boolean fireEvent = true;

	/**
	 * @param formToolkit
	 */
	public RadiosDialogField() {
		super();
	}

	/**
	 * this method must be called before create control
	 * 
	 * @param items
	 */
	public void setItems(String[] items) {
		this.items = items;
		button = new Button[this.items.length];
	}

	// ------- layout helpers

	/*
	 * @see DialogField#doFillIntoGrid
	 */
	public Control[] doFillIntoGrid(FormToolkit toolkit, Composite parent,
			int nColumns) {
		assertEnoughColumns(nColumns);

		Control requiredLabel = getRequiredLabelControl(toolkit, parent);
		requiredLabel.setLayoutData(gridDataForLabel(1));

		Control label = getLabelControl(toolkit, parent);
		label.setLayoutData(gridDataForLabel(1));

		group = getGroup(toolkit, parent);
		group.setLayoutData(gridDataForGroup(nColumns - 2));

		return new Control[] { requiredLabel, label, group };
	}

	/*
	 * @see DialogField#getNumberOfControls
	 */
	public int getNumberOfControls() {
		return 3;
	}

	protected static GridData gridDataForGroup(int span) {
		GridData gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		gd.grabExcessHorizontalSpace = false;
		gd.horizontalSpan = span;
		return gd;
	}

	// ------- focus methods

	/*
	 * @see DialogField#setFocus
	 */
	public boolean setFocus() {
		if (isOkToUse(group)) {
			group.setFocus();
		}
		return true;
	}

	// ------- ui creation

	public Composite getGroup(FormToolkit toolkit, Composite parent) {
		if (group == null) {
			assertCompositeNotNull(parent);
			if (toolkit != null) {
				group = toolkit.createComposite(parent);
			} else {
				group = new Composite(parent, SWT.NONE);
			}
			RowLayout layout = new RowLayout();
			layout.marginBottom = 0;
			group.setLayout(layout);
			for (int i = 0; i < items.length; i++) {
				if (toolkit != null) {
					button[i] = toolkit
							.createButton(group, items[i], SWT.RADIO);
				} else {
					button[i] = new Button(group, SWT.RADIO);
					button[i].setText(items[i]);
				}
				button[i].setData(INDEXKEY, new Integer(i));
				button[i].addSelectionListener(new SelectionAdapter() {
					public void widgetSelected(SelectionEvent e) {
						if (fireEvent) {
							selectIndex = ((Integer) e.widget.getData(INDEXKEY))
									.intValue();
							// FIXME: seemed will fire widgetSelected twice, one
							// for the deselect one,
							// one for the newly selected one. Need investigate.
							if (((Button) e.widget).getSelection()) {
								dialogFieldChangedAndApplied();
							}
						}
					}

				});
			}
		}
		return group;
	}

	// ------ enable / disable management

	/*
	 * @see DialogField#updateEnableState
	 */
	protected void updateEnableState() {
		super.updateEnableState();
		if (isOkToUse(group)) {
			group.setEnabled(isEnabled());
		}
	}

	// ------ text access

	/**
	 * Sets the text. Triggers a dialog-changed event.
	 */
	public void setSelectedIndex(int index) {
		selectIndex = index;
		if (isOkToUse(group)) {
			if (selectIndex >= 0 && selectIndex < items.length) {
				button[selectIndex].setSelection(true);
			} else {
				for (int i = 0; i < items.length; i++) {
					if (button[i].getSelection()) {
						button[i].setSelection(false);
					}
				}
			}
		}
		dialogFieldChangedAndApplied();
	}

	/**
	 * Sets the text without triggering a dialog-changed event.
	 */
	public void setSelectedIndexWithoutUpdate(int index) {
		selectIndex = index;
		if (isOkToUse(group)) {
			fireEvent = false;
			if (selectIndex >= 0 && selectIndex < items.length) {
				button[selectIndex].setSelection(true);
			} else {
				for (int i = 0; i < items.length; i++) {
					if (button[i].getSelection()) {
						button[i].setSelection(false);
					}
				}
			}
			fireEvent = true;
		}
	}

	public int getSelectedIndex() {
		return selectIndex;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jst.jsf.facesconfig.common.dialogfield.DialogField#handleGrabHorizontal()
	 */
	public void handleGrabHorizontal() {
		LayoutUtil.setGrabHorizontal(this.group, true);
	}
}
