/*******************************************************************************
 * Copyright (c) 2004, 2005 Sybase, Inc. and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sybase, Inc. - initial API and implementation
 *******************************************************************************/
package org.eclipse.jst.jsf.facesconfig.common.dialogfield;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.window.Window;
import org.eclipse.jst.jsf.facesconfig.common.CommonPlugin;
import org.eclipse.jst.jsf.facesconfig.common.dialogs.CommonResourceDialog;
import org.eclipse.jst.jsf.facesconfig.common.utils.PathUtil;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;


/**
 * This class provides a common resource choice DialogFiled. You must set a
 * no-null IProject instance to enable it working. You should give it a suffix
 * list if you want to choose files ended with specific file extensions. The
 * selection result will be a relative path based on the _referredFile's value
 * if it is set, or else based on the _project path.
 * 
 * The usage: Shell shell = new Shell(); IProject project = getProject();
 * ResourceButtonDialogField localeField = new
 * ResourceButtonDialogField(project);
 * localeField.setResourceDescription("image"); localeField.setSuffixs(new
 * String[]{"bmp","jpg","gif"});
 * localeField.setReferredFile(StructuredModelUtil.getFileFor(_element.getModel()));
 * 
 * @author zpzhang
 */
public class ResourceButtonDialogField extends StringButtonDialogField {
	private IProject project;

	private String[] suffixs;

	private String resourceDescription;

	private IFile referredFile;

	private boolean isWebPath = false;

	private IFolder folder;

	private String separator = "";

	public ResourceButtonDialogField(IProject project) {
		this(null, project);
		setStringButtonAdapter(new IStringButtonAdapter() {
			public void changeControlPressed(DialogField field) {
				String oldValue = getText();
				String newValue = browseButtonPressed();
				if (newValue != null && !newValue.equals(oldValue)) {
					setText(newValue);
				}
			}
		});
	}

	public ResourceButtonDialogField(IStringButtonAdapter adapter,
			IProject project) {
		super(adapter);
		this.project = project;
	}

	protected String browseButtonPressed() {
		Shell shell = getShell();
		CommonResourceDialog dialog = null;
		int style = "".equals(separator) ? SWT.NONE : SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL;
		if (folder != null) {
			dialog = new CommonResourceDialog(shell, folder, style);
		} else {
			dialog = new CommonResourceDialog(shell, project, style);
		}
		dialog.setTitle(CommonPlugin
				.getResourceString("DialogField.ResourceButton.SelectFile"));//$NON-NLS-1$
		dialog.setSuffixs(suffixs);
		dialog.setResourceDescription(resourceDescription);

		if (dialog.open() == Window.OK) {
			Object[] result = dialog.getResult();
			StringBuffer buffer = new StringBuffer();
			for (int i = 0; i < result.length; i++) {
				String newValue = null;
				IPath path = ((IFile) result[i]).getLocation();

				IPath referredPath = null;
				if (referredFile != null) {
					referredPath = referredFile.getLocation();
				} else {
					referredPath = project.getLocation();
				}
				if (this.isWebPath) {
					IFile selectedFile = ((IFile) result[i]);
					// String prjName = selectedFile.getProject().getName();
					// String filePath = selectedFile.getFullPath().toString();
					newValue = PathUtil.convertToWebPath(selectedFile.getFullPath());
				} else {
					newValue = PathUtil.convertToRelativePath(path.toString(),
							referredPath.toString());
				}
				buffer.append(newValue);
				buffer.append(separator);
			}
			if (buffer.length() > 0) {
				return buffer
						.substring(0, buffer.length() - separator.length());
			}

		}
		return null;
	}

	/**
	 * @return Returns the project.
	 */
	public IProject getProject() {
		return project;
	}

	/**
	 * @param project
	 *            The project to set.
	 */
	public void setProject(IProject project) {
		this.project = project;
	}

	/**
	 * @return Returns the relative folder.
	 */
	public IFolder getFolder() {
		return folder;
	}

	/**
	 * @param folder
	 *            The relative folder to set.
	 */
	public void setFolder(IFolder folder) {
		this.folder = folder;
	}

	/**
	 * @return Returns the referredFile.
	 */
	public IFile getReferredFile() {
		return referredFile;
	}

	/**
	 * @param referredFile
	 *            The referredFile to set.
	 */
	public void setReferredFile(IFile referredFile) {
		this.referredFile = referredFile;
	}

	/**
	 * @return Returns the resourceDescription.
	 */
	public String getResourceDescription() {
		return resourceDescription;
	}

	/**
	 * @param resourceDescription
	 *            The resourceDescription to set.
	 */
	public void setResourceDescription(String resourceDescription) {
		this.resourceDescription = resourceDescription;
	}

	/**
	 * @return Returns the suffixs.
	 */
	public String[] getSuffixs() {
		return suffixs;
	}

	/**
	 * @param suffixs
	 *            The suffixs to set.
	 */
	public void setSuffixs(String[] suffixs) {
		this.suffixs = suffixs;
	}

	/**
	 * set some special path to web path instead of relative path
	 * 
	 * @param isWebPath
	 */
	public void setWebPath(boolean isWebPath) {
		this.isWebPath = isWebPath;
	}

	public String getSeparator() {
		return separator;
	}

	public void setSeparator(String separator) {
		this.separator = separator;
	}
}