/*******************************************************************************
 * Copyright (c) 2006 Oracle Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Cameron Bateman/Oracle - initial API and implementation
 *    
 ********************************************************************************/

package org.eclipse.jst.jsf.validation.internal.el;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.jst.jsf.context.resolver.structureddocument.internal.provisional.IStructuredDocumentContextResolverFactory;
import org.eclipse.jst.jsf.context.resolver.structureddocument.internal.provisional.IWorkspaceContextResolver;
import org.eclipse.jst.jsf.context.structureddocument.internal.provisional.IStructuredDocumentContext;
import org.eclipse.jst.jsf.context.symbol.internal.provisional.IInstanceSymbol;
import org.eclipse.jst.jsf.context.symbol.internal.provisional.IPropertySymbol;
import org.eclipse.jst.jsf.context.symbol.internal.provisional.ISymbol;
import org.eclipse.jst.jsf.core.internal.types.BooleanLiteralType;
import org.eclipse.jst.jsf.core.internal.types.FloatLiteralType;
import org.eclipse.jst.jsf.core.internal.types.IntegerLiteralType;
import org.eclipse.jst.jsf.core.internal.types.LiteralType;
import org.eclipse.jst.jsf.core.internal.types.NullLiteralType;
import org.eclipse.jst.jsf.core.internal.types.SignatureBasedType;
import org.eclipse.jst.jsf.core.internal.types.StringLiteralType;
import org.eclipse.jst.jsf.core.internal.types.ValueType;
import org.eclipse.jst.jsf.designtime.internal.provisional.resolver.ISymbolContextResolver;
import org.eclipse.jst.jsf.designtime.internal.provisional.resolver.StructuredDocumentSymbolResolverFactory;
import org.eclipse.jst.jsf.validation.internal.el.diagnostics.ValidationMessageFactory;
import org.eclipse.jst.jsf.validation.internal.el.operators.BinaryOperator;
import org.eclipse.jst.jsf.validation.internal.el.operators.TernaryChoiceOperator;
import org.eclipse.jst.jsf.validation.internal.el.operators.UnaryOperator;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTAddExpression;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTAndExpression;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTChoiceExpression;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTEqualityExpression;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTExpression;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTFunctionInvocation;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTLiteral;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTMultiplyExpression;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTOperatorExpression;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTOrExpression;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTRelationalExpression;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTUnaryExpression;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTValue;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTValuePrefix;
import org.eclipse.jst.jsp.core.internal.java.jspel.ASTValueSuffix;
import org.eclipse.jst.jsp.core.internal.java.jspel.JSPELParser;
import org.eclipse.jst.jsp.core.internal.java.jspel.JSPELParserConstants;
import org.eclipse.jst.jsp.core.internal.java.jspel.JSPELParserVisitor;
import org.eclipse.jst.jsp.core.internal.java.jspel.SimpleNode;
import org.eclipse.jst.jsp.core.internal.java.jspel.Token;
import org.eclipse.wst.validation.internal.core.Message;
import org.eclipse.wst.validation.internal.provisional.core.IMessage;
import org.eclipse.wst.validation.internal.provisional.core.IReporter;
import org.eclipse.wst.validation.internal.provisional.core.IValidator;


class ASTSemanticValidator implements JSPELParserVisitor
{
	private final IFile								_targetFile;
	private final ASTExpression 					_expr;
	private final IStructuredDocumentContext		_context;
	private final ISymbolContextResolver			_symbolResolver;
	private final List								_messages;
	private final EvaluationTracker                 _tracker;
    
	ASTSemanticValidator(ASTExpression expr, IStructuredDocumentContext context)
	{
		final IWorkspaceContextResolver resolver = 
            IStructuredDocumentContextResolverFactory.INSTANCE.getWorkspaceContextResolver(context);
		if (resolver != null)
		{
			_targetFile = (IFile) resolver.getResource();
		}
		else
		{
			_targetFile = null;
		}
		_expr = expr;
		_context = context;
		_symbolResolver = StructuredDocumentSymbolResolverFactory.getInstance().getSymbolContextResolver(_context);
		_messages = new ArrayList();
        _tracker = new EvaluationTracker();
	}
	
	/**
	 * Performs the semantic validatino
	 */
	public void validate()
	{
		_expr.jjtAccept(this, _tracker);
	}
	
	public Object visit(ASTAddExpression node, Object data) 
	{
        performBinaryEvaluation(node, (EvaluationTracker)data);
        return data;
	}

	public Object visit(ASTAndExpression node, Object data) 
	{
        performBinaryEvaluation(node, (EvaluationTracker)data);
        return data;
	}

	public Object visit(ASTChoiceExpression node, Object data) 
    {
        if (node.jjtGetNumChildren() != 3)
        {
            throw new AssertionError("Binary operators should always have two sub-expressions");
        }
        
        // evaluate choice argument
        node.jjtGetChild(0).jjtAccept(this, data);
        final ValueType choiceArg = ((EvaluationTracker)data).getValueType();
        // evaluate when true argument
        node.jjtGetChild(1).jjtAccept(this, data);
        final ValueType whenTrueArg = ((EvaluationTracker)data).getValueType();
        //evaluate when false argument
        node.jjtGetChild(2).jjtAccept(this, data);
        final ValueType whenFalseArg = ((EvaluationTracker)data).getValueType();
        
        
        if (choiceArg != null && whenTrueArg != null && whenFalseArg != null)
        {
            final TernaryChoiceOperator operator = 
                new TernaryChoiceOperator();
            
            final Diagnostic diagnostic = operator.validate(choiceArg/* whenTrueArg, whenFalseArg*/);
            
            if (diagnostic.getSeverity() != Diagnostic.OK)
            {
                final Token firstToken = node.getFirstToken();
                final int offset = _context.getDocumentPosition() + firstToken.beginColumn - 1;
                final int length = node.getLastToken().endColumn - firstToken.beginColumn+1;
                final Message message = 
                    ValidationMessageFactory.createFromDiagnostic(diagnostic, 
                                                   offset, length, _targetFile);
                _messages.add(message);
            }
            
            ((EvaluationTracker)data).setType(operator.perform(choiceArg, whenTrueArg, whenFalseArg));
        }
        else
        {
            ((EvaluationTracker)data).setType(null);
        }
        
        return data;
	}

	public Object visit(ASTEqualityExpression node, Object data) {
        performBinaryEvaluation(node, (EvaluationTracker)data);
        return data;
    }

	public Object visit(ASTExpression node, Object data) {
		return node.childrenAccept(this, data);
	}

	public Object visit(ASTFunctionInvocation node, Object data) 
    {
	    // when we see a function invocation, null the type
        // we do not validate function invocations currently
        Object retVal = node.childrenAccept(this, data);
        ((EvaluationTracker)data).setType(null);
        return retVal;
	}

	public Object visit(ASTLiteral node, Object data) 
    {
        // note, there is an implicit assumption here that literals
        // are all terminals (leafs in the tree)
        if (node.jjtGetNumChildren() > 0)
        {
            throw new AssertionError("Literals should be terminal");
        }
        
        LiteralType type = null;
        
        Token  literalToken = node.getFirstToken();
        
        switch (literalToken.kind)
        {
            case JSPELParserConstants.STRING_LITERAL:
                type = new StringLiteralType(stripQuotes(literalToken.image));
            break;
            
            case JSPELParserConstants.INTEGER_LITERAL:
                type = new IntegerLiteralType(Long.parseLong(literalToken.image));
            break;
        
            case JSPELParserConstants.FLOATING_POINT_LITERAL:
                type = new FloatLiteralType(Double.parseDouble(literalToken.image));
            break;
            
            case JSPELParserConstants.FALSE:
                type = BooleanLiteralType.FALSE;
            break;
            
            case JSPELParserConstants.TRUE:
                type = BooleanLiteralType.TRUE;
            break;
            
            case JSPELParserConstants.NULL:
                type = NullLiteralType.SINGLETON;
            break;
            
            default:
                // do nothing... TODO:
        }

        ((EvaluationTracker)data).setType(type);
        return data;
	}

    private String stripQuotes(String stringLiteral)
    {
        if (stringLiteral.startsWith("'")
                || stringLiteral.startsWith("\""))
                        
        {
            if (stringLiteral.length() > 2)
            {
                // take 'literal' -> literal
                return stringLiteral.substring(1, stringLiteral.length()-1);
            }
            // if only two characters, then the empty string
            return "";
        }
        
        return stringLiteral;
    }
    
	public Object visit(ASTMultiplyExpression node, Object data) {
        performBinaryEvaluation(node, (EvaluationTracker)data);
        return data;
	}

	public Object visit(ASTOrExpression node, Object data) 
    {
	    performBinaryEvaluation(node, (EvaluationTracker)data);
        return data;
	}

	public Object visit(ASTRelationalExpression node, Object data) {
        performBinaryEvaluation(node, (EvaluationTracker)data);
        return data;
	}

	public Object visit(ASTUnaryExpression node, Object data) 
    {
        // assertion here is that this expression decomposes:
        // UnaryExpr -> Value
        // UnaryExpr -> UnaryOp UnaryExpression
        // since UnaryOp is a terminal (-,!,not,empty) node will
        // always have exactly one child
        node.childrenAccept(this, data);
        final SignatureBasedType type = ((EvaluationTracker)data).getType();
        
        if (type != null)
        {
            final Token  firstToken = node.getFirstToken();
            if (UnaryOperator.isUnaryOperator(firstToken))
            {
                if (type instanceof ValueType)
                {
                    final UnaryOperator unaryOp = UnaryOperator.createUnaryOperator(firstToken);
                    final Diagnostic diagnostic = unaryOp.validate((ValueType)type);
                    
                    if (diagnostic.getSeverity() != Diagnostic.OK)
                    {
                        final int offset = _context.getDocumentPosition() + firstToken.beginColumn - 1;
                        final int length = node.getLastToken().endColumn - firstToken.beginColumn+1;
                        final Message message = 
                            ValidationMessageFactory.createFromDiagnostic(diagnostic, offset, length, _targetFile);
                        _messages.add(message);
                    }
   
                    ((EvaluationTracker)data).
                            setType(unaryOp.performOperation ((ValueType)type));
                }
                // cannot apply operations to method bindings
                else
                {
                    final int offset = _context.getDocumentPosition() + 
                                    firstToken.beginColumn - 1;
                    final int length = node.getLastToken().endColumn - 
                                        firstToken.beginColumn+1;
                    final Message message =
                        ValidationMessageFactory.create_CANNOT_APPLY_OPERATOR_TO_METHOD_BINDING(offset, length, _targetFile);
                    _messages.add(message);
                }
            }
        }

        return data;
	}

	public Object visit(ASTValue node, final Object data) {
        ValueExpressionTracker  tracker = 
            new ValueExpressionTracker(_targetFile.getProject());
        
        ((EvaluationTracker)data).setValueTracker(tracker);
        
		node.childrenAccept(this, data);
        
        // now check the tracker.  If the last property in the expression
        // is non-null (i.e. the value has one or more suffices) then we
        // to very the leaf node (i.e. 'z' in #{x.y.z}) is more than just
        // an intermediate value used to get to other properties
        
        if (!tracker.isErrorFlag()
                && tracker.getCurMemberSymbol() instanceof IPropertySymbol
                && ((IPropertySymbol)tracker.getCurMemberSymbol()).isIntermediate())
        {
            final int problemStartOffset = tracker.getCurPropertySymbolOffset();
            final int length = tracker.getCurPropertySymbolLength();
            final Message message =
                ValidationMessageFactory.create_MEMBER_NOT_FOUND(problemStartOffset, 
                                         length, _targetFile, tracker.getCurMemberSymbol().getName(), tracker.getRootSymbol().getName());
            _messages.add(message);
        }
        // TODO: probably should eval type as we go
        else if (tracker.isRootValid() && !tracker.isErrorFlag())
        {
            final SignatureBasedType type = tracker.resolveType();
            if (type != null)
            {
                ((EvaluationTracker)data).setType(type);
            }
        }
            
        return data;
	}

	public Object visit(ASTValuePrefix node, final Object data) {
		ValueExpressionTracker  tracker = 
            ((EvaluationTracker) data).getValueTracker();
		
        if (!tracker.isErrorFlag())
        {
    		if (node.jjtGetNumChildren() == 0)
    		{
    			final Token token = node.getFirstToken();
    			final String image = token.image;
    			
    			final ISymbol symbol = _symbolResolver.getVariable(image);
    			
    			if (symbol == null)
    			{
                    final int problemStartOffset = 
                        _context.getDocumentPosition() + token.beginColumn - 1;
                    final int length = token.endColumn - token.beginColumn + 1;
                    final Message message =
                        ValidationMessageFactory.
                        create_VARIABLE_NOT_FOUND(problemStartOffset, length, 
                                                    _targetFile, image);
    				_messages.add(message);
    			}
    			else if (symbol instanceof IInstanceSymbol)
    			{
    				tracker.setRootSymbol((IInstanceSymbol) symbol);
    			}
    		}
        }

		return node.childrenAccept(this, data);
	}

	public Object visit(final ASTValueSuffix node, final Object data) 
    {
        final ValueExpressionTracker tracker = ((EvaluationTracker) data).getValueTracker();
        
        if (!tracker.isErrorFlag())
        {
    		final Token firstToken = node.getFirstToken();
    
    		if (node.jjtGetNumChildren() == 0
    				&& firstToken.kind == JSPELParserConstants.DOT)
    		{
    			final Token dotId = node.getLastToken();
    
    			if (tracker.isRootValid())
                {
                    final int startOffset = 
                        _context.getDocumentPosition() + dotId.beginColumn - 1;
                    final int length = dotId.endColumn - dotId.beginColumn + 1;
                    
    			    if (!tracker.isValidMemberOfCurrentSymbol(dotId.image))
    			    {
                        // TODO:
    			        final String ownerName = 
                            tracker.getCurMemberSymbol() != null ?
                                    tracker.getCurMemberSymbol().getName() :
                                        tracker.getRootSymbol().getName();

        				final Message message =
                            ValidationMessageFactory.create_MEMBER_NOT_FOUND(startOffset, length, _targetFile, dotId.image, ownerName);

        				_messages.add(message);
                        tracker.setErrorFlag(true);
                    }
                    else
                    {
                        tracker.setCurMemberSymbol(tracker.getNextPropertySymbol
                                (dotId.image), startOffset, length);
                    }
                }
    		}
        }
        
		return node.childrenAccept(this, data);
	}

	public Object visit(SimpleNode node, Object data) {
		return node.childrenAccept(this, data);
	}

	/**
	 * Copies stored messages into the validation reporter 
	 * @param validator
	 * @param reporter
	 */
	public void reportFindings(IValidator validator, IReporter reporter)
	{
		for (final Iterator it = _messages.iterator(); it.hasNext();)
		{
			reporter.addMessage(validator, (IMessage) it.next());
		}
	}
    
    private void performBinaryEvaluation(ASTOperatorExpression node, EvaluationTracker tracker)
    {
        if (node.jjtGetNumChildren() < 2)
        {
            throw new AssertionError("Binary operators should always have at least two sub-expressions");
        }
        else if (node.getOperatorTokens().size() != node.jjtGetNumChildren()-1)
        {
            throw new AssertionError("Binary operators should always have one operator token less than number of sub-expressions");
        }
        
        // evaluate left-most argument
        node.jjtGetChild(0).jjtAccept(this, tracker);
        ValueType curType = tracker.getValueType();
            
        for (int child = 1; child < node.jjtGetNumChildren(); child++)
        {
            // evaluate next argument running left-to-right
            node.jjtGetChild(child).jjtAccept(this, tracker);
            final ValueType secondType = tracker.getValueType();
            
            if (curType != null && secondType != null)
            {
                final BinaryOperator operator = 
                    BinaryOperator.getBinaryOperator((Token)node.getOperatorTokens().get(child-1));
                
                final Diagnostic diagnostic = operator.validate(curType, secondType);
                
                if (diagnostic.getSeverity() != Diagnostic.OK)
                {
                    final Token firstToken = node.getFirstToken();
                    final int offset = _context.getDocumentPosition() + firstToken.beginColumn - 1;
                    final int length = node.getLastToken().endColumn - firstToken.beginColumn+1;
                    final Message message = 
                        ValidationMessageFactory.createFromDiagnostic
                                (diagnostic, offset, length, _targetFile);
                    _messages.add(message);
                }
                
                curType = operator.performOperation(curType, secondType);
            }
        }
        
        tracker.setType(curType);
    }

	/**
	 * @return the list of generated message (may be empty if validate has not been called).
	 */
	public List getMessages()
	{
		// TODO: should we make this throw exception if validate has
		// not been called?
		return _messages;
	}
    
    /**
     * @return the value type of fully resolved expression
     * or null if not resolved (or could not be resolved)
     */
    public SignatureBasedType getExpressionType()
    {
        return _tracker.getType();
    }
    
    /**
     * @param args
     * @throws IOException
     */
    public static void main(String[] args) throws IOException
    {
        String elText = "";
        int nextCharacter;
        
        while(((nextCharacter = System.in.read()) != -1))
        {
            char nextChar = (char) nextCharacter;
            
            if (nextChar == '\n')
            {
                try
                {
                    JSPELParser parser = JSPELParser.createParser(elText);
                    ASTExpression expr = parser.Expression();
                    expr.dump("");
                }
                catch (Throwable t)
                {
                    t.printStackTrace(System.err);
                }
                
                elText = "";
            }
            else
            {
                elText += nextChar;
            }   
        }
    }
}
