/*******************************************************************************
 * Copyright (c) 2006 Oracle Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Cameron Bateman/Oracle - initial API and implementation
 *    
 ********************************************************************************/

package org.eclipse.jst.jsf.validation.internal.el;

import org.eclipse.core.resources.IProject;
import org.eclipse.jst.jsf.context.symbol.internal.provisional.IInstanceSymbol;
import org.eclipse.jst.jsf.context.symbol.internal.provisional.IMethodSymbol;
import org.eclipse.jst.jsf.context.symbol.internal.provisional.IObjectSymbol;
import org.eclipse.jst.jsf.context.symbol.internal.provisional.ISymbol;
import org.eclipse.jst.jsf.context.symbol.internal.provisional.ITypeDescriptor;
import org.eclipse.jst.jsf.core.internal.types.IAssignable;
import org.eclipse.jst.jsf.core.internal.types.MethodType;
import org.eclipse.jst.jsf.core.internal.types.SignatureBasedType;
import org.eclipse.jst.jsf.core.internal.types.ValueType;
import org.eclipse.jst.jsf.designtime.internal.provisional.DesignTimeApplicationManager;
import org.eclipse.jst.jsf.designtime.internal.provisional.el.AbstractDTMethodResolver;
import org.eclipse.jst.jsf.designtime.internal.provisional.el.AbstractDTPropertyResolver;

class ValueExpressionTracker
{
    private final IProject              _project;

    private IInstanceSymbol				_rootSymbol;
    private boolean                     _isRootValid;
    
    private ISymbol                     _curMemberSymbol;
    private int                         _curPropertySymbolOffset;
    private int                         _curPropertySymbolLength;
    
    private boolean                     _errorFlag; // set to true when an error is found before a value expr is completely processed.
                                                    // used as a hint to stop processing when an error is found

	/**
	 * Constructor
	 * 
	 * @param project
	 */
	public ValueExpressionTracker(IProject project) {
		super();
        _project = project;
	}
	
	/**
	 * Set the root symbol of a value expression
	 * 
	 * @param rootSymbol
	 */
	public void setRootSymbol(IInstanceSymbol rootSymbol)
	{
		_isRootValid = rootSymbol != null ? true : false;
		_rootSymbol = rootSymbol;
	}

	/**
	 * @return whether or not the root is set and valid
	 */
	public boolean isRootValid() 
	{
		return _isRootValid;
	}

	/**
	 * @return the root symbol or null if not set
	 */
	public ISymbol getRootSymbol() {
		return _rootSymbol;
	}
	

    /**
     * @param name 
     * @return the next property symbol for the name'd property
     */
    public ISymbol getNextPropertySymbol(final String name)
    {
        final ISymbol curSymbol = 
            _curMemberSymbol != null ? 
                    (ISymbol) _curMemberSymbol : _rootSymbol;
                    
        if (curSymbol instanceof IObjectSymbol)
        {
            return getMemberSymbol((IObjectSymbol) curSymbol, name);
        }
        
        return null;
    }

    
    /**
     * @param name
     * @return true if symbol name is a valid member of the current prop
     */
    public boolean isValidMemberOfCurrentSymbol(final String name)
    {
        if (_curMemberSymbol == null)
        {
            return isValidMemberOfRoot(name);
        }
        else if (_curMemberSymbol instanceof IObjectSymbol)
        {
            return getMemberSymbol((IObjectSymbol) _curMemberSymbol, name) != null;
        }
        else
        {
            return false;
        }
    }

    /**
     * @return the current property of a ValueSuffix chain
     */
    public ISymbol getCurMemberSymbol() 
    {
        return _curMemberSymbol;
    }

    /**
     * @param memberSymbol 
     * @param offset 
     * @param length 
     */
    public void setCurMemberSymbol(final ISymbol memberSymbol, int offset, int length) 
    {
        _curMemberSymbol = memberSymbol;
        _curPropertySymbolOffset = offset;
        _curPropertySymbolLength = length;
    }
    
    /**
     * @return the offset of the current property symbol or 0 if no current property symbol
     */
    public int getCurPropertySymbolOffset()
    {
        return _curPropertySymbolOffset;
    }
    
    /**
     * @return the length of the current property symbol or 0 if no current property symbol
     */
    public int getCurPropertySymbolLength()
    {
        return _curPropertySymbolLength;
    }

    /**
     * @return true if error flag is set.
     */
    public boolean isErrorFlag() {
        return _errorFlag;
    }

    /**
     * Property is immutable once set to true.
     * 
     * @param flag
     */
    public void setErrorFlag(boolean flag) 
    {
        if (!_errorFlag)
        {
            _errorFlag = flag;
        }
    }
    
    /**
     * @return the type of this value expression or null if it cannot be
     * resolved
     * @throws IllegalStateException if root symbol is not yet set
     */
    public SignatureBasedType resolveType()
    {
        if (!_isRootValid || _rootSymbol == null)
        {
            throw new IllegalStateException("Cannot resolve type without root symbol");
        }
        
        // if there's been an error resolving the expression semantics, don't
        // bother trying to determine type
        if (_errorFlag)
        {
            return null;
        }
        
        if (_curMemberSymbol instanceof IMethodSymbol)
        {
            return new MethodType(_curMemberSymbol.getName(),
                                  ((IMethodSymbol)_curMemberSymbol).getSignature());
        }

        IObjectSymbol  objectSymbol = null;
        
        // no properties, so resolve root's type
        if (_curMemberSymbol == null)
        {
            objectSymbol = _rootSymbol;
        }
        else 
        {
            objectSymbol = (IObjectSymbol) _curMemberSymbol;
        }

        final ITypeDescriptor typeDesc = objectSymbol.getTypeDescriptor();
        int   assignability = 0;
        assignability |= objectSymbol.isReadable() 
                            ? IAssignable.ASSIGNMENT_TYPE_RHS
                            : 0;
        assignability |= objectSymbol.isWritable() 
                            ? IAssignable.ASSIGNMENT_TYPE_LHS
                            : 0;
        final String typeDescString = typeDesc.getTypeSignature();
        final String[] superTypes = (String[]) 
            typeDesc.getSuperTypeSignatures().toArray(new String[0]);
        final String[] interfaceTypes = (String[]) 
            typeDesc.getInterfaceTypeSignatures().toArray(new String[0]);

        return new ValueType(typeDescString, superTypes, interfaceTypes,
                assignability);
    }
    
    /**
     * @param name
     * @return whether name is a member of the root
     */
    private boolean isValidMemberOfRoot(final String name)
    {
        // if the root symbol isn't yet initialized, then this call is
        // meaningless
        if (_rootSymbol == null
                || name == null
                || "".equals(name))
        {
            return false;
        }

        return getMemberSymbol(_rootSymbol, name) != null;
    }
    
    private ISymbol getMemberSymbol(final IObjectSymbol symbol, final String name)
    {
        ISymbol  memberSymbol = getPropertySymbol(symbol, name);

        if (memberSymbol != null)
        {
            return memberSymbol;
        }

        memberSymbol = getMethodSymbol(symbol, name);
        
        // otherwise, see if it's a valid method
        if (memberSymbol != null)
        {
            return memberSymbol;
        }
        
        // if not a property or method, then not a valid member
        return null;
    }
    
    /**
     * @param symbol
     * @param name
     * @return the property symbol called name relative to 'symbol' or null
     * if one doesn't exist
     */
    private ISymbol getPropertySymbol(final ISymbol symbol, final String name)
    {
        DesignTimeApplicationManager dtAppMgr =
            DesignTimeApplicationManager.getInstance(_project);
        
        AbstractDTPropertyResolver resolver = dtAppMgr.getPropertyResolver();
        
        if (resolver != null)
        {
            return resolver.getProperty(symbol,name);
        }
        
        Activator.log("Error acquiring property resolver", new Throwable());
        return null;
    }

    private IMethodSymbol getMethodSymbol(final IObjectSymbol symbol, final String name)
    {
        DesignTimeApplicationManager dtAppMgr =
            DesignTimeApplicationManager.getInstance(_project);
        
        AbstractDTMethodResolver resolver = dtAppMgr.getMethodResolver();
        
        if (resolver != null)
        {
            return resolver.getMethod(symbol, name);
        }
        
        Activator.log("Error acquiring property resolver", new Throwable());
        return null;

    }
}
