/*******************************************************************************
 * Copyright (c) 2006 Oracle Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Cameron Bateman/Oracle - initial API and implementation
 *    
 ********************************************************************************/

package org.eclipse.jst.jsf.validation.internal.el.diagnostics;

import org.eclipse.core.resources.IFile;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.osgi.util.NLS;
import org.eclipse.wst.validation.internal.core.Message;
import org.eclipse.wst.validation.internal.provisional.core.IMessage;

/**
 * Creates configured validation Message objects.
 * 
 * @author cbateman
 *
 */
public final class ValidationMessageFactory 
{
    
    /**
     * Method bindings cannot participate in expressions involving operators
     * Per JSP 1.1 spec section 5.2.1
     * 
     * @param offset
     * @param length
     * @param targetFile
     * @return a configured message
     */
    public static Message create_CANNOT_APPLY_OPERATOR_TO_METHOD_BINDING(int offset, int length, IFile targetFile)
    {
        final Message message = new MyLocalizedMessage(IMessage.HIGH_SEVERITY, 
                Messages.CANNOT_APPLY_OPERATORS_TO_MB, 
                targetFile, 0);
        
        message.setOffset(offset);
        message.setLength(length);
        return message;
    }

    /**
     * @param offset
     * @param length
     * @param targetFile
     * @param curMemberSymbol
     * @param owningMember
     * @return a configured message
     */
    public static Message create_MEMBER_NOT_FOUND(int offset, int length, IFile targetFile, String curMemberSymbol, String owningMember)
    {
        final Message message = new MyLocalizedMessage(IMessage.NORMAL_SEVERITY, 
                NLS.bind(Messages.VM_PROP_NAME_NOT_FOUND, curMemberSymbol, owningMember), 
                targetFile,
                0);
        message.setOffset(offset);
        message.setLength(length);
        message.setAttribute("name", curMemberSymbol);
        return message;
    }

    /**
     * @param offset
     * @param length
     * @param targetFile
     * @param variableName
     * @return a configured message
     */
    public static Message create_VARIABLE_NOT_FOUND(int offset, int length, 
                                          IFile targetFile, String variableName)
    {
        final Message message = new MyLocalizedMessage(IMessage.NORMAL_SEVERITY, 
                NLS.bind(Messages.VM_ROOT_NAME_NOT_FOUND, variableName),
                targetFile, 0);
        message.setOffset(offset);
        message.setLength(length);
        message.setAttribute("name", variableName);
        return message;
    }

    /**
     * @param offset
     * @param length
     * @param file
     * @return a configured message
     */
    public static Message create_MISSING_CLOSING_EXPR_BRACKET(int offset, int length, IFile file)
    {
        final Message message = 
            new MyLocalizedMessage(IMessage.HIGH_SEVERITY, 
                                   Messages.MISSING_CLOSING_EXPR_BRACKET, 
                                   file, 0);
        message.setOffset(offset);
        message.setLength(length);
        return message;
    }
    
    /**
     * @param offset
     * @param length
     * @param file
     * @return a configured message
     */
    public static Message create_GENERAL_SYNTAX_ERROR(int offset, int length, IFile file)
    {
        final Message message = 
            new MyLocalizedMessage(IMessage.NORMAL_SEVERITY,
                                   Messages.GENERAL_SYNTAX_ERROR, 
                                   file, 0);
        message.setOffset(offset);
        message.setLength(length);
        return message;
    }

    /**
     * @param offset
     * @param length
     * @param file
     * @return a configured message
     */
    public static Message create_EMPTY_EL_EXPRESSION(int offset, int length, IFile file)
    {
        final Message message = 
            new MyLocalizedMessage(IMessage.NORMAL_SEVERITY,
                                   Messages.EMPTY_EL_EXPRESSION, 
                                   file, 0);
        message.setOffset(offset);
        message.setLength(length);

        return message;
    }
    
    /**
     * @param diagnostic
     * @param offset
     * @param length
     * @param file
     * @return a configured message
     */
    public static Message createFromDiagnostic(final Diagnostic diagnostic, int offset, int length, IFile file)
    {
        final Message message =
            new MyLocalizedMessage(
                    convertSeverity(diagnostic), 
                    diagnostic.getMessage(), 
                    file, 
                    diagnostic.getCode());
        
        message.setOffset(offset);
        message.setLength(length);
        
        return message;
    }
    
    /**
     * @param diagnostic
     * @return a Message severity equivilent to diagnostic.getSeverity()
     */
    private static int convertSeverity(Diagnostic diagnostic)
    {
        switch (diagnostic.getSeverity())
        {
            case Diagnostic.ERROR:
                return IMessage.HIGH_SEVERITY;
            case Diagnostic.WARNING:
                return IMessage.NORMAL_SEVERITY;
            case Diagnostic.INFO:
                return IMessage.LOW_SEVERITY;
                
            default:
                return IMessage.LOW_SEVERITY;
        }
    }
    
    

    private ValidationMessageFactory()
    {
        // do nothing; no external instantiation
    }
}
